import { Controller, UsePipes } from '@nestjs/common';
import { CommandBus } from '@nestjs/cqrs';
import { GrpcMethod, RpcException } from '@nestjs/microservices';
import { AggregateID } from '@mobicoop/ddd-library';
import { IdResponse } from '@mobicoop/ddd-library';
import { RpcExceptionCode } from '@mobicoop/ddd-library';
import { RpcValidationPipe } from '@mobicoop/ddd-library';
import { CreateUserRequestDto } from './dtos/create-user.request.dto';
import { CreateUserCommand } from '@modules/user/core/application/commands/create-user/create-user.command';
import {
  EmailAlreadyExistsException,
  PhoneAlreadyExistsException,
  UserAlreadyExistsException,
} from '@modules/user/core/domain/user.errors';
import { GRPC_SERVICE_NAME } from '@src/app.constants';

@UsePipes(
  new RpcValidationPipe({
    whitelist: false,
    forbidUnknownValues: false,
  }),
)
@Controller()
export class CreateUserGrpcController {
  constructor(private readonly commandBus: CommandBus) {}

  @GrpcMethod(GRPC_SERVICE_NAME, 'Create')
  async create(
    createUserRequestDto: CreateUserRequestDto,
  ): Promise<IdResponse> {
    try {
      const aggregateID: AggregateID = await this.commandBus.execute(
        new CreateUserCommand({
          firstName: createUserRequestDto.firstName,
          lastName: createUserRequestDto.lastName,
          email: createUserRequestDto.email,
          phone: createUserRequestDto.phone,
          userId: createUserRequestDto.id,
        }),
      );
      return new IdResponse(aggregateID);
    } catch (error: any) {
      if (
        error instanceof UserAlreadyExistsException ||
        error instanceof EmailAlreadyExistsException ||
        error instanceof PhoneAlreadyExistsException
      )
        throw new RpcException({
          code: RpcExceptionCode.ALREADY_EXISTS,
          message: error.message,
        });
      throw new RpcException({
        code: RpcExceptionCode.UNKNOWN,
        message: error.message,
      });
    }
  }
}
